package com.example.venue_reservation_service.controller;

import com.example.venue_reservation_service.annotation.Log;
import com.example.venue_reservation_service.domain.User;
import com.example.venue_reservation_service.dto.*;
import com.example.venue_reservation_service.service.UserService;
import com.example.venue_reservation_service.vo.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jakarta.annotation.Resource;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

@RestController
@RequestMapping("/user")
@Api("用户接口管理")
@CrossOrigin
public class UserController {

    @Resource
    private UserService userService;

    @GetMapping("/test")
    public Result test(){
        return Result.ok("test").message("success");
    }

    @ApiOperation("login")
    @GetMapping("/login")
    public Result getOpenId(@RequestParam("code")String code){
        return userService.getOpenId(code);
    }

    @ApiOperation("管理员登录")
    @PostMapping("/admin")
//    @Log(operationType = "POST", operationDesc = "场馆管理员登录，用户账号为#{#body.account}")
    public Result adminLogin(@RequestBody AdminDTO dto){
        return userService.adminLogin(dto);
    }

    @ApiOperation("用户个人信息完善")
    @PostMapping("/update")
    public Result update(@RequestBody UserInfo userInfo){
        return userService.userUpdate(userInfo);
    }

    @ApiOperation("获取用户列表")
    @PostMapping("/list")
    public Result queryUserList(@RequestBody UserDTO dto){
        return userService.getUserList(dto);
    }

    @ApiOperation("校验用户身份信息")
    @PostMapping("/check")
    public Result check(@RequestBody CheckDTO dto){
        return userService.userCheck(dto);
    }

    @ApiOperation("获取用户的认证信息")
    @GetMapping("/detail")
    public Result get(@RequestParam("id") Integer userId){
        return userService.getSchool(userId);
    }

    @ApiOperation("充值")
    @PostMapping("/recharge")
    public Result recharge(@RequestParam("id") Integer userId,
                           @RequestParam("amount") Double amount){
        return userService.userRecharge(userId, amount);
    }

    @ApiOperation("用户补充手机号及邮箱")
    @PostMapping("/phone")
    public Result check(@RequestBody PhoneDTO dto){
        return userService.checkPhone(dto);
    }

    @ApiOperation("获取用户的数量")
    @GetMapping("/num")
    public Result num(){
        return userService.useNum();
    }

    @ApiOperation("获取管理员信息")
    @PostMapping("/admininfo")
    public Result admininfo(@RequestBody PageDTO dto){
        return userService.admininfo(dto);
    }

    @ApiOperation("超级管理员添加管理员信息")
    @PostMapping("/create")
    public Result create(@RequestBody User user){
        return userService.createAdmin(user);
    }

    @ApiOperation("移除管理员信息")
    @GetMapping("/remove")
    public Result remove(@RequestParam("userId") Integer userId){
        return userService.removeAdmin(userId);
    }

    @ApiOperation("修改账号密码")
    @PostMapping("/passwd")
    public Result passwd(@RequestBody PasswdDTO dto){
        return userService.passwd(dto);
    }

    @ApiOperation("用户头像上传")
    @PostMapping("/upload")
    public Result upload(@RequestParam("id") Integer userId, @RequestParam("image")MultipartFile file){
        return userService.upload(userId, file);
    }

    @ApiOperation("用户信息获取")
    @GetMapping("/info/{id}")
    public Result info(@PathVariable("id") Integer userId){
        return userService.getUserInfo(userId);
    }
}
